@@ -42,7 +42,7 @@ public:
 
         auto curl = curl_easy_init();
         if (!curl) {
-            CurlException("Failed to create a new curl handle.");
+            throw CurlException("Failed to create a new curl handle.");
         }
 
         if (m_maxbytes > 0) {
@@ -52,9 +52,18 @@ public:
             }
         }
 
-        curl_easy_setopt(curl, CURLOPT_URL, url.c_str());
-        curl_easy_setopt(curl, CURLOPT_WRITEFUNCTION, &write_data);
-        curl_easy_setopt(curl, CURLOPT_WRITEDATA, this);
+        CURLcode rv = curl_easy_setopt(curl, CURLOPT_URL, url.c_str());
+        if (rv != CURLE_OK) {
+            throw CurlException("Failed to set CURLOPT_URL.");
+        }
+        rv = curl_easy_setopt(curl, CURLOPT_WRITEFUNCTION, &write_data);
+        if (rv != CURLE_OK) {
+            throw CurlException("Failed to set CURLOPT_WRITEFUNCTION.");
+        }
+        rv = curl_easy_setopt(curl, CURLOPT_WRITEDATA, this);
+        if (rv != CURLE_OK) {
+            throw CurlException("Failed to set CURLOPT_WRITEDATA.");
+        }
 
         auto res = curl_easy_perform(curl);
         if (res != CURLE_OK) {
@@ -172,18 +181,26 @@ picojson::value::object find_key_id(const picojson::value json, const std::strin
         throw JsonException("Metadata resource is missing 'keys' array value");
     }
     auto keys_array = iter->second.get<picojson::array>();
-    for (auto &key : keys_array) {
-        if (!key.is<picojson::object>()) {continue;}
+    if (kid.empty()) {
+        if (keys_array.size() != 1) {
+            throw JsonException("Key ID empty but multiple keys published.");
+        }
+        auto &key = keys_array.at(0);
+        return key.get<picojson::object>();
+    } else {
+        for (auto &key : keys_array) {
+            if (!key.is<picojson::object>()) {continue;}
 
-        auto key_obj = key.get<picojson::object>();
-        iter = key_obj.find("kid");
-        if (iter == key_obj.end() || (!iter->second.is<std::string>())) {continue;}
+            auto key_obj = key.get<picojson::object>();
+            iter = key_obj.find("kid");
+            if (iter == key_obj.end() || (!iter->second.is<std::string>())) {continue;}
 
-        std::string cur_kid = iter->second.get<std::string>();
+            std::string cur_kid = iter->second.get<std::string>();
 
-        if (cur_kid == kid) {return key_obj;}
+            if (cur_kid == kid) {return key_obj;}
+        }
+        throw JsonException("Key ID is not published by the issuer.");
     }
-    throw JsonException("Key ID is not published by the issuer.");
 }
 
 
@@ -410,7 +427,7 @@ Validator::get_public_keys_from_web(const std::string &issuer, picojson::value &
     keys = json_obj;
 
     next_update = now + 600;
-    expires = now + 4*3600;
+    expires = now + 4*24*3600;
 }
 
 void
